@@ -1,6 +1,6 @@
 /*
  * sar, sadc, sadf, mpstat and iostat common routines.
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -28,7 +28,6 @@
 #include <dirent.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#include <sys/param.h>	/* for HZ */
 
 /*
  * For PAGE_SIZE (which may be itself a call to getpagesize()).
@@ -49,6 +48,8 @@
 #define _(string) (string)
 #endif
 
+/* Get HZ from the kernel */
+unsigned int hz;
 
 /*
  ***************************************************************************
@@ -65,7 +66,37 @@ void print_version(void)
 
 /*
  ***************************************************************************
- * Get current date or time
+ * Get date and time and take into account <ENV_TIME_DEFTM> variable
+ ***************************************************************************
+ */
+time_t get_time(struct tm *loc_time)
+{
+   time_t timer;
+   struct tm *ltm;
+   static int utc = 0;
+   char *e;
+
+   if (!utc) {
+      /* Read environment variable value once */
+      if ((e = getenv(ENV_TIME_DEFTM)) != NULL)
+	 utc = !strcmp(e, K_UTC);
+      utc++;
+   }
+
+   time(&timer);
+   if (utc ==  2)
+      ltm = gmtime(&timer);
+   else
+      ltm = localtime(&timer);
+
+   *loc_time = *ltm;
+   return timer;
+}
+
+
+/*
+ ***************************************************************************
+ * Get local date and time
  ***************************************************************************
  */
 time_t get_localtime(struct tm *loc_time)
@@ -83,12 +114,13 @@ time_t get_localtime(struct tm *loc_time)
 
 /*
  ***************************************************************************
- * Return the number of processors used on the machine
- * (0 means one proc, 1 means two proc, etc.)
+ * Return the number of processors used on the machine:
+ * 0: one proc and non SMP kernel
+ * 1: one proc and SMP kernel, or SMP machine with all but one offlined CPU
+ * 2: two proc...
  * As far as I know, there are two possibilities for this:
  * 1) Use /proc/stat (this is what we are doing here) or
  * 2) Use /proc/cpuinfo
- * (I haven't heard of a better method to guess it...)
  ***************************************************************************
  */
 int get_cpu_nr(unsigned int max_nr_cpus)
@@ -111,24 +143,14 @@ int get_cpu_nr(unsigned int max_nr_cpus)
       }
    }
 
-   /*
-    * cpu_nr initial value: -1
-    * If cpu_nr < 0 then there is only one proc.
-    * If cpu_nr > 0 then this is an SMP machine.
-    * If cpu_nr = 0 then there is only one proc but this is a Linux 2.2 SMP or
-    * Linux 2.4 kernel.
-    */
-   if (cpu_nr < 0)
-      cpu_nr = 0;
-
-   if (cpu_nr >= max_nr_cpus) {
+   if ((cpu_nr + 1) > max_nr_cpus) {
       fprintf(stderr, _("Cannot handle so many processors!\n"));
       exit(1);
    }
 
    fclose(fp);
 
-   return cpu_nr;
+   return (cpu_nr + 1);
 }
 
 
@@ -335,7 +357,7 @@ inline void print_gal_header(struct tm *loc_time, char *sysname, char *release,
    char cur_date[64];
    char *e;
 
-   if (((e = getenv(TM_FMT_VAR)) != NULL) && !strcmp(e, K_ISO))
+   if (((e = getenv(ENV_TIME_FMT)) != NULL) && !strcmp(e, K_ISO))
       strftime(cur_date, sizeof(cur_date), "%Y-%m-%d", loc_time);
    else
       strftime(cur_date, sizeof(cur_date), "%x", loc_time);
@@ -419,6 +441,22 @@ int get_kb_shift(void)
 }
 
 
+/*
+ ***************************************************************************
+ * Get number of clock ticks per second
+ ***************************************************************************
+ */
+void get_HZ(void)
+{
+   long ticks;
+
+   if ((ticks = sysconf(_SC_CLK_TCK)) < 0)
+      perror("sysconf");
+
+   hz = (unsigned int) ticks;
+}
+
+
 /*
  ***************************************************************************
  * Handle overflow conditions properly for counters which are read as
