@@ -1,6 +1,6 @@
 /*
  * iostat: report CPU and I/O statistics
- * (C) 1998-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1998-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -32,9 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
-#include <sys/param.h>	/* for HZ */
 
-#include "version.h"
 #include "iostat.h"
 #include "common.h"
 #include "ioconf.h"
@@ -63,11 +61,8 @@ int dlist_idx = 0;
 long interval = 0;
 unsigned char timestamp[64];
 
-/*
- * Nb of processors on the machine.
- * A value of 1 means two procs...
- */
-int cpu_nr = -1;
+/* Nb of processors on the machine */
+int cpu_nr = 0;
 
 
 /*
@@ -77,13 +72,11 @@ int cpu_nr = -1;
  */
 void usage(char *progname)
 {
-   fprintf(stderr, _("sysstat version %s\n"
-		   "(C) Sebastien Godard\n"
-	           "Usage: %s [ options... ] [ <interval> [ <count> ] ]\n"
+   fprintf(stderr, _("Usage: %s [ options... ] [ <interval> [ <count> ] ]\n"
 		   "Options are:\n"
 		   "[ -c | -d ] [ -k | -m ] [ -t ] [ -V ] [ -x ]\n"
 		   "[ <device> [ ... ] | ALL ] [ -p [ <device> | ALL ] ]\n"),
-	   VERSION, progname);
+	   progname);
    exit(1);
 }
 
@@ -341,7 +334,7 @@ void read_proc_stat(int curr, int flags)
    unsigned long v_tmp[4];
    unsigned int v_major, v_index;
    struct io_stats *st_iodev_tmp[4];
-   unsigned long long cc_idle, cc_iowait;
+   unsigned long long cc_idle, cc_iowait, cc_steal;
    unsigned long long cc_user, cc_nice, cc_system, cc_hardirq, cc_softirq;
 
 
@@ -367,16 +360,18 @@ void read_proc_stat(int curr, int flags)
 	  * Some fields are only presnt in 2.6 kernels.
 	  */
 	 comm_stats[curr].cpu_iowait = 0;	/* For pre 2.6 kernels */
+	 comm_stats[curr].cpu_steal = 0;
 	 cc_hardirq = cc_softirq = 0;
 	 /* CPU counters became unsigned long long with kernel 2.6.5 */
-	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu",
+	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu",
 	        &(comm_stats[curr].cpu_user), &(comm_stats[curr].cpu_nice),
 		&(comm_stats[curr].cpu_system), &(comm_stats[curr].cpu_idle),
-		&(comm_stats[curr].cpu_iowait), &cc_hardirq, &cc_softirq);
+		&(comm_stats[curr].cpu_iowait), &cc_hardirq, &cc_softirq,
+		&(comm_stats[curr].cpu_steal));
 
 	 /*
 	  * Time spent in system mode also includes time spent servicing
-	  * interrupts and softirqs.
+	  * hard interrupts and softirqs.
 	  */
 	 comm_stats[curr].cpu_system += cc_hardirq + cc_softirq;
 	
@@ -388,22 +383,23 @@ void read_proc_stat(int curr, int flags)
 	                           comm_stats[curr].cpu_nice +
 	                           comm_stats[curr].cpu_system +
 	                           comm_stats[curr].cpu_idle +
-	                           comm_stats[curr].cpu_iowait;
+	                           comm_stats[curr].cpu_iowait +
+	    			   comm_stats[curr].cpu_steal;
       }
 
-      else if ((!strncmp(line, "cpu0", 4)) && cpu_nr) {
+      else if ((!strncmp(line, "cpu0", 4)) && (cpu_nr > 1)) {
 	 /*
 	  * Read CPU line for proc#0 (if available).
 	  * Useful to compute uptime reduced to one processor on SMP machines,
 	  * with fewer risks to get an overflow...
 	  */
-	 cc_iowait = cc_hardirq = cc_softirq = 0;
-	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu",
+	 cc_iowait = cc_hardirq = cc_softirq = cc_steal = 0;
+	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu",
 		&cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait,
-		&cc_hardirq, &cc_softirq);
+		&cc_hardirq, &cc_softirq, &cc_steal);
 	 comm_stats[curr].uptime0 = cc_user + cc_nice + cc_system +
 	    			    cc_idle + cc_iowait +
-	    			    cc_hardirq + cc_softirq;
+	    			    cc_hardirq + cc_softirq + cc_steal;
       }
 
       else if (DISPLAY_EXTENDED(flags) || HAS_DISKSTATS(flags) ||
@@ -774,13 +770,14 @@ void read_ppartitions_stat(int curr, int flags)
  */
 void write_cpu_stat(int curr, unsigned long long itv)
 {
-   printf("avg-cpu:  %%user   %%nice %%system %%iowait   %%idle\n");
+   printf("avg-cpu:  %%user   %%nice %%system %%iowait  %%steal   %%idle\n");
 
-   printf("         %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n\n",
+   printf("         %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n\n",
 	  ll_sp_value(comm_stats[!curr].cpu_user,   comm_stats[curr].cpu_user,   itv),
 	  ll_sp_value(comm_stats[!curr].cpu_nice,   comm_stats[curr].cpu_nice,   itv),
 	  ll_sp_value(comm_stats[!curr].cpu_system, comm_stats[curr].cpu_system, itv),
 	  ll_sp_value(comm_stats[!curr].cpu_iowait, comm_stats[curr].cpu_iowait, itv),
+	  ll_sp_value(comm_stats[!curr].cpu_steal,  comm_stats[curr].cpu_steal,  itv),
 	  (comm_stats[curr].cpu_idle < comm_stats[!curr].cpu_idle) ?
 	  0.0 :
 	  ll_sp_value(comm_stats[!curr].cpu_idle, comm_stats[curr].cpu_idle, itv));
@@ -796,15 +793,17 @@ void write_stat_header(int flags, int *fctr)
 {
    if (DISPLAY_EXTENDED(flags)) {
       /* Extended stats */
-      printf("Device:    rrqm/s wrqm/s   r/s   w/s  rsec/s  wsec/s");
+      printf("Device:         rrqm/s   wrqm/s   r/s   w/s");
       if (DISPLAY_MEGABYTES(flags)) {
 	 printf("    rMB/s    wMB/s");
 	 *fctr = 2048;
       }
-      else {
+      else if (DISPLAY_KILOBYTES(flags)) {
 	 printf("    rkB/s    wkB/s");
 	 *fctr = 2;
       }
+      else
+	 printf("   rsec/s   wsec/s");
       printf(" avgrq-sz avgqu-sz   await  svctm  %%util\n");
    }
    else {
@@ -846,14 +845,14 @@ void write_ext_stat(int curr, unsigned long long itv, int flags, int fctr,
     * sometimes negative...
     */
    nr_ios = (ioi->rd_ios - ioj->rd_ios) + (ioi->wr_ios - ioj->wr_ios);
-   tput   = ((double) nr_ios) * HZ / itv;
+   tput = ((double) nr_ios) * HZ / itv;
    util = S_VALUE(ioj->tot_ticks, ioi->tot_ticks, itv);
-   svctm  = tput ? util / tput : 0.0;
+   svctm = tput ? util / tput : 0.0;
    /*
     * kernel gives ticks already in milliseconds for all platforms
     * => no need for further scaling.
     */
-   await  = nr_ios ?
+   await = nr_ios ?
       ((ioi->rd_ticks - ioj->rd_ticks) + (ioi->wr_ticks - ioj->wr_ticks)) /
       nr_ios : 0.0;
 
@@ -864,19 +863,17 @@ void write_ext_stat(int curr, unsigned long long itv, int flags, int fctr,
    if ((ioi->wr_sectors < ioj->wr_sectors) && (ioj->wr_sectors <= 0xffffffff))
       wr_sec &= 0xffffffff;
 
-   arqsz  = nr_ios ? (rd_sec + wr_sec) / nr_ios : 0.0;
+   arqsz = nr_ios ? (rd_sec + wr_sec) / nr_ios : 0.0;
 
-   printf("%-10s", shi->name);
+   printf("%-13s", shi->name);
    if (strlen(shi->name) > 10)
       printf("\n          ");
-   /*       rrq/s wrq/s   r/s   w/s  rsec  wsec   r?B   w?B  rqsz  qusz await svctm %util */
-   printf(" %6.2f %6.2f %5.2f %5.2f %7.2f %7.2f %8.2f %8.2f %8.2f %8.2f %7.2f %6.2f %6.2f\n",
+   /*       rrq/s wrq/s   r/s   w/s  rsec  wsec  rqsz  qusz await svctm %util */
+   printf(" %8.2f %8.2f %5.2f %5.2f %8.2f %8.2f %8.2f %8.2f %7.2f %6.2f %6.2f\n",
 	  S_VALUE(ioj->rd_merges, ioi->rd_merges, itv),
 	  S_VALUE(ioj->wr_merges, ioi->wr_merges, itv),
 	  S_VALUE(ioj->rd_ios, ioi->rd_ios, itv),
 	  S_VALUE(ioj->wr_ios, ioi->wr_ios, itv),
-	  ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv),
-	  ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv),
 	  ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
 	  ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
 	  arqsz,
@@ -987,7 +984,7 @@ int write_stat(int curr, int flags, struct tm *loc_time)
       /* Display CPU utilization */
       write_cpu_stat(curr, itv);
 
-   if (cpu_nr) {
+   if (cpu_nr > 1) {
       /* On SMP machines, reduce itv to one processor (see note above) */
       if (!comm_stats[!curr].uptime0)
 	 itv = comm_stats[curr].uptime0;
@@ -1137,6 +1134,9 @@ int main(int argc, char **argv)
    init_nls();
 #endif
 
+   /* Get HZ */
+   get_HZ();
+
    /* Allocate structures for device list */
    if (argc > 1)
       salloc_dev_list(argc - 1);
@@ -1199,7 +1199,10 @@ int main(int argc, char **argv)
 	       flags |= I_D_EXTENDED;	/* Display extended stats */
 	       break;
 
-	     case 'V':			/* Print usage and exit	*/
+	     case 'V':			/* Print version number and exit */
+	       print_version();
+	       break;
+	
 	     default:
 	       usage(argv[0]);
 	    }
