@@ -1,6 +1,6 @@
 /*
  * sar/sadc: report system activity
- * (C) 1999-2005 by Sebastien Godard (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien Godard (sysstat <at> wanadoo.fr)
  */
 
 #ifndef _SA_H
@@ -18,7 +18,7 @@
  * System activity daily file magic number
  * (will vary when file format changes)
  */
-#define SA_MAGIC	0x2167
+#define SA_MAGIC	0x2169
 
 
 /* Define activities */
@@ -87,7 +87,6 @@
 #define K_NFS	"NFS"
 #define K_NFSD	"NFSD"
 #define K_SOCK	"SOCK"
-#define K_FULL	"FULL"
 
 /* S_= sar/sadc/sadf - F_= Flag */
 #define S_F_ALL_PROC      	0x0001
@@ -102,6 +101,7 @@
 #define S_F_HAS_DISKSTATS	0x0200
 #define S_F_FILE_LCK		0X0400
 #define S_F_PER_PROC		0x0800
+#define S_F_X_ALL		0x1000
 
 #define WANT_ALL_PROC(m)	(((m) & S_F_ALL_PROC) == S_F_ALL_PROC)
 #define WANT_SA_ROTAT(m)	(((m) & S_F_SA_ROTAT) == S_F_SA_ROTAT)
@@ -115,13 +115,14 @@
 #define HAS_DISKSTATS(m)	(((m) & S_F_HAS_DISKSTATS) == S_F_HAS_DISKSTATS)
 #define FILE_LOCKED(m)		(((m) & S_F_FILE_LCK) == S_F_FILE_LCK)
 #define WANT_PER_PROC(m)	(((m) & S_F_PER_PROC) == S_F_PER_PROC)
+#define WANT_ALL_PIDS(m)	(((m) & S_F_X_ALL) == S_F_X_ALL)
 
 /* Output formats (O_= Output)  */
-#define S_O_NONE		0
 #define S_O_HDR_OPTION		1
 #define S_O_PPC_OPTION		2
 #define S_O_DB_OPTION		3
 #define S_O_XML_OPTION		4
+#define S_O_DBD_OPTION		5
 
 /* Files */
 #define PROC		"/proc"
@@ -154,6 +155,7 @@
 
 /* Maximum number of processes that can be monitored simultaneously */
 #define MAX_PID_NR	256
+
 /* Maximum length of network interface name */
 #define MAX_IFACE_LEN	IFNAMSIZ
 /*
@@ -221,7 +223,11 @@ struct file_hdr {
    unsigned int   sa_irqcpu			__attribute__ ((packed));
    /* Number of disks */
    unsigned int   sa_nr_disk			__attribute__ ((packed));
-   /* Number of processors: 1 means two proc */
+   /* Number of processors:
+    * 0 means 1 proc and non SMP machine
+    * 1 means 1 proc and SMP machine
+    * 2 means two proc, etc.
+    */
    unsigned int   sa_proc 			__attribute__ ((packed));
    /* Number of serial lines: 2 means two lines (ttyS00 and ttyS01) */
    unsigned int   sa_serial 			__attribute__ ((packed));
@@ -241,6 +247,11 @@ struct file_hdr {
    unsigned char  sa_day			__attribute__ ((packed));
    unsigned char  sa_month			__attribute__ ((packed));
    unsigned char  sa_year			__attribute__ ((packed));
+   /*
+    * Size of a long integer. Useful to know the architecture on which
+    * the datafile was created.
+    */
+   char		  sa_sizeof_long		__attribute__ ((packed));
    /* Operating system name */
    char           sa_sysname[UTSNAME_LEN]	__attribute__ ((packed));
    /* Machine hostname */
@@ -263,6 +274,7 @@ struct file_stats {
    unsigned long long cpu_system		__attribute__ ((aligned (16)));
    unsigned long long cpu_idle			__attribute__ ((aligned (16)));
    unsigned long long cpu_iowait		__attribute__ ((aligned (16)));
+   unsigned long long cpu_steal			__attribute__ ((aligned (16)));
    unsigned long long irq_sum			__attribute__ ((aligned (16)));
    /* --- LONG --- */
    /* Time stamp (number of seconds since the epoch) */
@@ -344,6 +356,7 @@ struct stats_one_cpu {
    unsigned long long per_cpu_user		__attribute__ ((aligned (16)));
    unsigned long long per_cpu_nice		__attribute__ ((aligned (16)));
    unsigned long long per_cpu_system		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_steal		__attribute__ ((aligned (16)));
    unsigned long long pad			__attribute__ ((aligned (16)));
 };
 
@@ -533,8 +546,8 @@ extern void	    prep_file_for_reading(int *, char *, struct file_hdr *,
 extern void	    get_itv_value(struct file_stats *, struct file_stats *,
 				  unsigned int, unsigned long long *,
 				  unsigned long long *);
-extern void	    print_report_hdr(unsigned short, unsigned int,
-				     struct tm *, struct file_hdr *);
+extern void	    print_report_hdr(unsigned int, struct tm *,
+				     struct file_hdr *);
 extern int	    sa_fread(int, void *, int, int);
 extern void	    salloc_cpu_array(struct stats_one_cpu * [], unsigned int);
 extern void	    salloc_disk_array(struct disk_stats * [], int);
@@ -543,5 +556,8 @@ extern void	    salloc_irqcpu_array(struct stats_irq_cpu * [],
 extern void	    salloc_net_dev_array(struct stats_net_dev * [],
 					 unsigned int);
 extern void	    salloc_serial_array(struct stats_serial * [], int);
+extern void	    set_default_file(struct tm *, char *);
+extern void	    set_hdr_loc_time(unsigned int, struct tm *,
+				     struct file_hdr *);
 
 #endif  /* _SA_H */
