@@ -1,6 +1,6 @@
 /*
  * sadc: system activity data collector
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -46,10 +46,10 @@
 #define _(string) (string)
 #endif
 
-/* Nb of processors on the machine. A value of 1 means two processors */
-int cpu_nr = -1;
+/* Nb of processors on the machine */
+int cpu_nr = 0;
 unsigned int serial_nr = 0, iface_nr = 0, irqcpu_nr = 0, disk_nr = 0;
-int pid_nr = 0;
+int pid_idx = 0, pid_nr = 0;
 unsigned int sadc_actflag;
 long interval = 0;
 int kb_shift = 0;
@@ -159,11 +159,11 @@ int save_pid(unsigned long pid)
 {
    int i = 0;
 
-   while ((i < pid_nr) && (all_pids[i] != pid))
+   while ((i < pid_idx) && (all_pids[i] != pid))
       i++;
    /* PID not found: insert it if possible */
-   if ((i == pid_nr) && (pid_nr < MAX_PID_NR))
-      all_pids[pid_nr++] = pid;
+   if ((i == pid_idx) && (pid_idx < MAX_PID_NR))
+      all_pids[pid_idx++] = pid;
 
    return i;
 }
@@ -326,7 +326,7 @@ void sa_sys_init(unsigned int *flags)
 {
    /* How many processors on this machine ? */
    if ((cpu_nr = get_cpu_nr(NR_CPUS)) > 0)
-      SREALLOC(st_cpu, struct stats_one_cpu, STATS_ONE_CPU_SIZE * (cpu_nr + 1));
+      SREALLOC(st_cpu, struct stats_one_cpu, STATS_ONE_CPU_SIZE * cpu_nr);
 
    /* Get serial lines that support accounting */
    if ((serial_nr = get_serial_lines())) {
@@ -334,10 +334,10 @@ void sa_sys_init(unsigned int *flags)
       SREALLOC(st_serial, struct stats_serial, STATS_SERIAL_SIZE * serial_nr);
    }
    /* Get number of interrupts available per processor */
-   if (cpu_nr > 0) {
+   if (cpu_nr) {
       if ((irqcpu_nr = get_irqcpu_nb(NR_IRQS)))
 	 SREALLOC(st_irq_cpu, struct stats_irq_cpu,
-		  STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr);
+		  STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr);
    }
    else
       /* IRQ per processor are not provided by sadc on UP machines */
@@ -420,20 +420,21 @@ void setup_file_hdr(int fd, size_t *file_stats_size)
    memset(&file_hdr, 0, FILE_HDR_SIZE);
 
    /* Then get current date */
-   file_hdr.sa_ust_time = get_localtime(&loc_time);
+   file_hdr.sa_ust_time = get_time(&loc_time);
 
    /* Ok, now fill the header */
    file_hdr.sa_actflag = sadc_actflag;
-   file_hdr.sa_magic   = SA_MAGIC;
+   file_hdr.sa_magic = SA_MAGIC;
    file_hdr.sa_st_size = FILE_STATS_SIZE;
-   file_hdr.sa_day     = loc_time.tm_mday;
-   file_hdr.sa_month   = loc_time.tm_mon;
-   file_hdr.sa_year    = loc_time.tm_year;
-   file_hdr.sa_proc    = cpu_nr;
-   file_hdr.sa_nr_pid  = pid_nr;
-   file_hdr.sa_serial  = serial_nr;
-   file_hdr.sa_irqcpu  = irqcpu_nr;
-   file_hdr.sa_iface   = iface_nr;
+   file_hdr.sa_day = loc_time.tm_mday;
+   file_hdr.sa_month = loc_time.tm_mon;
+   file_hdr.sa_year = loc_time.tm_year;
+   file_hdr.sa_sizeof_long = sizeof(long);
+   file_hdr.sa_proc = cpu_nr;
+   file_hdr.sa_nr_pid = pid_nr;
+   file_hdr.sa_serial = serial_nr;
+   file_hdr.sa_irqcpu = irqcpu_nr;
+   file_hdr.sa_iface = iface_nr;
    if (GET_DISK(sadc_actflag))
       file_hdr.sa_nr_disk = disk_nr;
    else
@@ -482,7 +483,7 @@ void write_dummy_record(int ofd, size_t file_stats_size, unsigned int *flags)
    file_stats.record_type = R_DUMMY;
 
    /* Save time */
-   file_stats.ust_time = get_localtime(&loc_time);
+   file_stats.ust_time = get_time(&loc_time);
 
    file_stats.hour   = loc_time.tm_hour;
    file_stats.minute = loc_time.tm_min;
@@ -498,12 +499,13 @@ void write_dummy_record(int ofd, size_t file_stats_size, unsigned int *flags)
  ***************************************************************************
  * Write stats.
  * NB: sadc provides all the stats, including:
- * -> CPU utilization per processor (on SMP machines only)
- * -> IRQ per processor (on SMP machines only)
+ * -> CPU utilization per processor (*)
+ * -> IRQ per processor (*)
  * -> number of each IRQ (if -I option passed to sadc), including APIC
  *    interrupts sources
  * -> device stats for sar -d (kernels 2.4 and newer only, and only if
  *    -d option passed to sadc)
+ * (*): on SMP machines only, even if there is only one available proc.
  ***************************************************************************
  */
 void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
@@ -518,8 +520,8 @@ void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
    }
    if ((nb = write(ofd, &file_stats, file_stats_size)) != file_stats_size)
       p_write_error();
-   if (cpu_nr > 0) {
-      if ((nb = write(ofd, st_cpu, STATS_ONE_CPU_SIZE * (cpu_nr + 1))) != (STATS_ONE_CPU_SIZE * (cpu_nr + 1)))
+   if (cpu_nr) {
+      if ((nb = write(ofd, st_cpu, STATS_ONE_CPU_SIZE * cpu_nr)) != (STATS_ONE_CPU_SIZE * cpu_nr))
 	 p_write_error();
    }
    if (GET_ONE_IRQ(sadc_actflag)) {
@@ -536,8 +538,8 @@ void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
 	 p_write_error();
    }
    if (irqcpu_nr) {
-      if ((nb = write(ofd, st_irq_cpu, STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr))
-	  != (STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr))
+      if ((nb = write(ofd, st_irq_cpu, STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr))
+	  != (STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr))
 	 p_write_error();
    }
    if (iface_nr) {
@@ -673,7 +675,7 @@ void open_ofile(int *ofd, char ofile[], size_t *file_stats_size, unsigned int *f
 	 if (file_hdr.sa_irqcpu != irqcpu_nr) {
 	    irqcpu_nr = file_hdr.sa_irqcpu;
 	    SREALLOC(st_irq_cpu, struct stats_irq_cpu,
-		     STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr);
+		     STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr);
 	 }
 	 if (file_hdr.sa_nr_disk != disk_nr) {
 	    if (!file_hdr.sa_nr_disk)
@@ -719,20 +721,19 @@ void read_proc_stat(unsigned int flags)
 	  * to one processor to avoid rounding problems.
 	  */
 	 file_stats.cpu_iowait = 0;	/* For pre 2.5 kernels */
-	 cc_hardirq = cc_softirq = cc_steal = 0;
+	 file_stats.cpu_steal = 0;
+	 cc_hardirq = cc_softirq = 0;
 	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu",
 		&(file_stats.cpu_user),   &(file_stats.cpu_nice),
 		&(file_stats.cpu_system), &(file_stats.cpu_idle),
 		&(file_stats.cpu_iowait), &cc_hardirq, &cc_softirq,
-		&cc_steal);
+		&(file_stats.cpu_steal));
 
 	 /*
 	  * Time spent in system mode also includes time spent
 	  * servicing hard interrrupts and softirqs.
-	  * Time spent in idle mode also include 'steal' time.
 	  */
 	 file_stats.cpu_system += cc_hardirq + cc_softirq;
-	 file_stats.cpu_idle += cc_steal;
 	
 	 /*
 	  * Compute the uptime of the system in jiffies (1/100ths of a second
@@ -746,34 +747,32 @@ void read_proc_stat(unsigned int flags)
 	  */
 	 file_stats.uptime = file_stats.cpu_user   + file_stats.cpu_nice +
 	                     file_stats.cpu_system + file_stats.cpu_idle +
-	                     file_stats.cpu_iowait;
+	                     file_stats.cpu_iowait + file_stats.cpu_steal;
       }
 
       else if (!strncmp(line, "cpu", 3)) {
-	 if (cpu_nr > 0) {
+	 if (cpu_nr) {
 	    /*
 	     * Read the number of jiffies spent in the different modes
 	     * (user, nice, etc) for current proc.
 	     * This is done only on SMP machines.
-	     * Warning: st_cpu_i struct is _not_ allocated even if the kernel
-	     * has SMP support enabled.
 	     */
-	    cc_iowait = 0;	/* For pre 2.5 kernels */
-	    cc_hardirq = cc_softirq = cc_steal = 0;
+	    cc_iowait = cc_steal = 0;	/* For pre 2.5 kernels */
+	    cc_hardirq = cc_softirq = 0;
 	    sscanf(line + 3, "%d %llu %llu %llu %llu %llu %llu %llu %llu",
 		   &proc_nb,
 		   &cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait,
 		   &cc_hardirq, &cc_softirq, &cc_steal);
 	    cc_system += cc_hardirq + cc_softirq;
-	    cc_idle += cc_steal;
 	
-	    if (proc_nb <= cpu_nr) {
+	    if (proc_nb < cpu_nr) {
 	       st_cpu_i = st_cpu + proc_nb;
 	       st_cpu_i->per_cpu_user   = cc_user;
 	       st_cpu_i->per_cpu_nice   = cc_nice;
 	       st_cpu_i->per_cpu_system = cc_system;
 	       st_cpu_i->per_cpu_idle   = cc_idle;
 	       st_cpu_i->per_cpu_iowait = cc_iowait;
+	       st_cpu_i->per_cpu_steal  = cc_steal;
 	    }
 	    /* else:
 	     * Additional CPUs have been dynamically registered in /proc/stat.
@@ -783,7 +782,7 @@ void read_proc_stat(unsigned int flags)
 	    if (!proc_nb)
 	       /* Compute uptime reduced to one proc using proc#0 */
 	       file_stats.uptime0 = cc_user + cc_nice + cc_system +
-	       			    cc_idle + cc_iowait;
+	       			    cc_idle + cc_iowait + cc_steal;
 	 }
       }
 
@@ -1038,13 +1037,16 @@ void read_proc_vmstat(void)
  * Read stats from /proc/<pid>/stat
  ***************************************************************************
  */
-void read_pid_stat(void)
+void read_pid_stat(unsigned int flags)
 {
    FILE *fp;
    int pid;
    static char filename[24];
 
-   for (pid = 0; pid < pid_nr; pid++) {
+   if (WANT_ALL_PIDS(flags))
+      get_pid_list();
+
+   for (pid = 0; pid < pid_idx; pid++) {
 
       if (!all_pids[pid])
 	 continue;
@@ -1153,7 +1155,7 @@ void read_interrupts_stat(void)
 	    p = st_irq_cpu + irq;
 	    sscanf(line, "%3u", &(p->irq));
 	
-	    for (cpu = 0; cpu <= cpu_nr; cpu++) {
+	    for (cpu = 0; cpu < cpu_nr; cpu++) {
 	       p = st_irq_cpu + cpu * irqcpu_nr + irq;
 	       /*
 		* No need to set (st_irq_cpu + cpu * irqcpu_nr)->irq:
@@ -1387,9 +1389,9 @@ void read_net_nfs_stat(void)
 		&(file_stats.nfs_rpccnt), &(file_stats.nfs_rpcretrans));
 	
       else if (!strncmp(line, "proc3", 5))
-	 sscanf(line + 5, "%*u %u %*u %*u %u %*u %u %u",
+	 sscanf(line + 5, "%*u %*u %u %*u %*u %u %*u %u %u",
 		&(file_stats.nfs_getattcnt), &(file_stats.nfs_accesscnt),
-		&(file_stats.nfs_writecnt), &(file_stats.nfs_readcnt));
+		&(file_stats.nfs_readcnt), &(file_stats.nfs_writecnt));
    }
 
    fclose(fp);
@@ -1425,9 +1427,9 @@ void read_net_nfsd_stat(void)
 		&(file_stats.nfsd_rpccnt), &(file_stats.nfsd_rpcbad));
 	
       else if (!strncmp(line, "proc3", 5))
-	 sscanf(line + 5, "%*u %u %*u %*u %u %*u %u %u",
+	 sscanf(line + 5, "%*u %*u %u %*u %*u %u %*u %u %u",
 		&(file_stats.nfsd_getattcnt), &(file_stats.nfsd_accesscnt),
-		&(file_stats.nfsd_writecnt), &(file_stats.nfsd_readcnt));
+		&(file_stats.nfsd_readcnt), &(file_stats.nfsd_writecnt));
    }
 
    fclose(fp);
@@ -1585,7 +1587,7 @@ void read_stats(unsigned int *flags)
    /* else disks are in /proc/stat */
 
    if (pid_nr)
-       read_pid_stat();
+       read_pid_stat(*flags);
     if (serial_nr)
       read_serial_stat();
    if (irqcpu_nr)
@@ -1625,7 +1627,7 @@ void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
 	 file_stats.record_type = R_STATS;
 
       /* Save time */
-      file_stats.ust_time = get_localtime(loc_time);
+      file_stats.ust_time = get_time(loc_time);
       file_stats.hour   = loc_time->tm_hour;
       file_stats.minute = loc_time->tm_min;
       file_stats.second = loc_time->tm_sec;
@@ -1685,10 +1687,7 @@ void rw_sa_stat_loop(unsigned int *flags, long count, struct tm *loc_time,
       /* Rotate activity file if necessary */
       if (WANT_SA_ROTAT(*flags)) {
 	 /* The user specified '-' as the filename to use */
-	 get_localtime(loc_time);
-	 snprintf(new_ofile, MAX_FILE_LEN,
-		  "%s/sa%02d", SA_DIR, loc_time->tm_mday);
-	 new_ofile[MAX_FILE_LEN - 1] = '\0';
+	 set_default_file(loc_time, new_ofile);
 
 	 if (strcmp(ofile, new_ofile))
 	    do_sa_rotat = TRUE;
@@ -1769,8 +1768,7 @@ int main(int argc, char **argv)
 	 sadc_actflag |= A_PID;
 	 if (argv[++opt]) {
 	    if (!strcmp(argv[opt], K_ALL)) {
-	       /* Get PID list */
-	       get_pid_list();
+	       flags |= S_F_X_ALL;
 	       set_pflag(strcmp(argv[opt - 1], "-x"), 0);
 	       continue;	/* Next option */
 	    }
@@ -1792,10 +1790,7 @@ int main(int argc, char **argv)
 	    stdfd = -1;	/* Don't write to STDOUT */
 	    if (!strcmp(argv[opt], "-")) {
 	       /* File name set to '-' */
-	       get_localtime(&loc_time);
-	       snprintf(ofile, MAX_FILE_LEN,
-			"%s/sa%02d", SA_DIR, loc_time.tm_mday);
-	       ofile[MAX_FILE_LEN - 1] = '\0';
+	       set_default_file(&loc_time, ofile);
 	       flags |= S_F_SA_ROTAT;
 	    }
 	    else if (!strncmp(argv[opt], "-", 1))
@@ -1838,13 +1833,19 @@ int main(int argc, char **argv)
 
    /* -x and -X options ignored when writing to a file */
    if (ofile[0]) {
-      pid_nr = 0;
+      pid_idx = 0;
+      flags &= ~S_F_X_ALL;
       sadc_actflag &= ~A_PID;
    }
    else
       /* -L option ignored when writing to STDOUT */
       flags &= ~S_F_L_OPTION;
 
+   if (WANT_ALL_PIDS(flags))
+      pid_nr = MAX_PID_NR;
+   else
+      pid_nr = pid_idx;
+
    /* Init structures according to machine architecture */
    sa_sys_init(&flags);
 
