@@ -1,6 +1,6 @@
 /*
  * sar: report system activity
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -25,7 +25,6 @@
 #include <unistd.h>
 #include <time.h>
 #include <errno.h>
-#include <sys/param.h>	/* for HZ */
 
 #include "sa.h"
 #include "common.h"
@@ -67,7 +66,6 @@ short dis_hdr = -1;
 int pid_nr = 0;
 char *args[MAX_ARGV_NR];
 
-
 /*
  ***************************************************************************
  * Print usage and exit
@@ -80,7 +78,7 @@ void usage(char *progname)
 	           "[ -A ] [ -b ] [ -B ] [ -c ] [ -d ] [ -i <interval> ] [ -p ] [ -q ]\n"
 		   "[ -r ] [ -R ] [ -t ] [ -u ] [ -v ] [ -V ] [ -w ] [ -W ] [ -y ]\n"
 		   "[ -I { <irq> | SUM | ALL | XALL } ] [ -P { <cpu> | ALL } ]\n"
-		   "[ -n { DEV | EDEV | NFS | NFSD | SOCK | FULL } ]\n"
+		   "[ -n { DEV | EDEV | NFS | NFSD | SOCK | ALL } ]\n"
 		   "[ -x { <pid> | SELF | ALL } ] [ -X { <pid> | SELF | ALL } ]\n"
 	           "[ -o [ <filename> ] | -f [ <filename> ] ]\n"
 		   "[ -s [ <hh:mm:ss> ] ] [ -e [ <hh:mm:ss> ] ]\n"),
@@ -154,8 +152,8 @@ void salloc_pid(int nr_pid)
  */
 void allocate_structures(int stype)
 {
-   if (file_hdr.sa_proc > 0)
-      salloc_cpu_array(st_cpu, file_hdr.sa_proc + 1);
+   if (file_hdr.sa_proc)
+      salloc_cpu_array(st_cpu, file_hdr.sa_proc);
    if ((stype == USE_SADC) &&
        (GET_PID(sar_actflag) || GET_CPID(sar_actflag))) {
       pid_nr = file_hdr.sa_nr_pid;
@@ -164,7 +162,7 @@ void allocate_structures(int stype)
    if (file_hdr.sa_serial)
       salloc_serial_array(st_serial, file_hdr.sa_serial);
    if (file_hdr.sa_irqcpu)
-      salloc_irqcpu_array(st_irq_cpu, file_hdr.sa_proc + 1, file_hdr.sa_irqcpu);
+      salloc_irqcpu_array(st_irq_cpu, file_hdr.sa_proc, file_hdr.sa_irqcpu);
    if (file_hdr.sa_iface)
       salloc_net_dev_array(st_net_dev, file_hdr.sa_iface);
    if (file_hdr.sa_nr_disk)
@@ -184,12 +182,12 @@ void check_smp_option(unsigned int cpu_nr)
 {
    unsigned int j = 0, i;
 
-   if (cpu_nr == 0) {
+   if (!cpu_nr) {
       fprintf(stderr, _("Not an SMP machine...\n"));
       exit(1);
    }
 
-   for (i = (cpu_nr + 1); i < NR_CPUS; i++)
+   for (i = cpu_nr; i < NR_CPUS; i++)
       j |= cpu_bitmap[i >> 3] & (1 << (i & 0x07));
 
    if (j) {
@@ -211,7 +209,7 @@ void prep_smp_option(unsigned int cpu_nr)
 
    if (WANT_PER_PROC(flags)) {
       if (WANT_ALL_PROC(flags))
-	 for (i = cpu_nr + 1; i < ((NR_CPUS >> 3) + 1) << 3; i++)
+	 for (i = cpu_nr; i < ((NR_CPUS >> 3) + 1) << 3; i++)
 	    /*
 	     * Reset every bit for proc > cpu_nr
 	     * (only done when -P ALL entered on the command line)
@@ -232,8 +230,6 @@ void set_loc_time(short curr)
 {
    struct tm *ltm;
 
-   /* NOTE: loc_time structure must have been init'ed before! */
-
    /* Check if option -t was specified on the command line */
    if (PRINT_TRUE_TIME(flags)) {
       /* -t */
@@ -308,7 +304,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
    if (GET_CPU(act)) {
       if (dis)
 	 printf("\n%-11s       CPU     %%user     %%nice   %%system   "
-		"%%iowait     %%idle\n",
+		"%%iowait    %%steal     %%idle\n",
 		prev_string);
 
       if (!WANT_PER_PROC(flags) ||
@@ -316,11 +312,12 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 
 	 printf("%-11s       all", curr_string);
 
-	 printf("    %6.2f    %6.2f    %6.2f    %6.2f",
+	 printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f",
 		ll_sp_value(fsj->cpu_user, fsi->cpu_user, g_itv),
 		ll_sp_value(fsj->cpu_nice, fsi->cpu_nice, g_itv),
 		ll_sp_value(fsj->cpu_system, fsi->cpu_system, g_itv),
-		ll_sp_value(fsj->cpu_iowait, fsi->cpu_iowait, g_itv));
+		ll_sp_value(fsj->cpu_iowait, fsi->cpu_iowait, g_itv),
+		ll_sp_value(fsj->cpu_steal, fsi->cpu_steal, g_itv));
 
 	 printf("    %6.2f\n",
 		fsi->cpu_idle < fsj->cpu_idle ? /* Handle buggy kernels */
@@ -333,7 +330,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	    *sci = st_cpu[curr],
 	    *scj = st_cpu[prev];
 	
-	 for (i = 0; i <= file_hdr.sa_proc; i++, sci++, scj++) {
+	 for (i = 0; i < file_hdr.sa_proc; i++, sci++, scj++) {
 	    if (cpu_bitmap[i >> 3] & (1 << (i & 0x07))) {
 
 	       printf("%-11s       %3d", curr_string, i);
@@ -341,11 +338,12 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	       /* Recalculate itv for current proc */
 	       pc_itv = get_per_cpu_interval(sci, scj);
 	
-	       printf("    %6.2f    %6.2f    %6.2f    %6.2f",
+	       printf("    %6.2f    %6.2f    %6.2f    %6.2f    %6.2f",
 		      ll_sp_value(scj->per_cpu_user, sci->per_cpu_user, pc_itv),
 		      ll_sp_value(scj->per_cpu_nice, sci->per_cpu_nice, pc_itv),
 		      ll_sp_value(scj->per_cpu_system, sci->per_cpu_system, pc_itv),
-		      ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv));
+		      ll_sp_value(scj->per_cpu_iowait, sci->per_cpu_iowait, pc_itv),
+		      ll_sp_value(scj->per_cpu_steal, sci->per_cpu_steal, pc_itv));
 
 	       printf("    %6.2f\n",
 		      sci->per_cpu_idle < scj->per_cpu_idle ?
@@ -547,7 +545,7 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	 printf("\n");
       }
 
-      for (k = 0; k <= file_hdr.sa_proc; k++) {
+      for (k = 0; k < file_hdr.sa_proc; k++) {
 	 if (cpu_bitmap[k >> 3] & (1 << (k & 0x07))) {
 
 	    printf("%-11s  %3d", curr_string, k);
@@ -610,7 +608,6 @@ void write_stats_core(short prev, short curr, short dis, char *prev_string,
 	    continue;
 	 j = check_iface_reg(&file_hdr, st_net_dev, curr, prev, i);
 	 sndj = st_net_dev[prev] + j;
-	
 	 printf("%-11s %9s", curr_string, sndi->interface);
 	
 	 printf(" %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f %9.2f\n",
@@ -1028,15 +1025,15 @@ void write_stats_startup(short curr)
    file_stats[!curr].minute      = file_stats[curr].minute;
    file_stats[!curr].second      = file_stats[curr].second;
    file_stats[!curr].ust_time    = file_stats[curr].ust_time;
-   if (file_hdr.sa_proc > 0)
-      memset(st_cpu[!curr], 0, STATS_ONE_CPU_SIZE * (file_hdr.sa_proc + 1));
+   if (file_hdr.sa_proc)
+      memset(st_cpu[!curr], 0, STATS_ONE_CPU_SIZE * file_hdr.sa_proc);
    memset(interrupts[!curr], 0, STATS_ONE_IRQ_SIZE);
    if (pid_nr)
       memset (pid_stats[!curr][0], 0, PID_STATS_SIZE * pid_nr);
    if (file_hdr.sa_serial)
       memset(st_serial[!curr], 0, STATS_SERIAL_SIZE * file_hdr.sa_serial);
    if (file_hdr.sa_irqcpu)
-      memset(st_irq_cpu[!curr], 0, STATS_IRQ_CPU_SIZE * (file_hdr.sa_proc + 1) * file_hdr.sa_irqcpu);
+      memset(st_irq_cpu[!curr], 0, STATS_IRQ_CPU_SIZE * file_hdr.sa_proc * file_hdr.sa_irqcpu);
    if (file_hdr.sa_iface)
       memset(st_net_dev[!curr], 0, STATS_NET_DEV_SIZE * file_hdr.sa_iface);
    if (file_hdr.sa_nr_disk)
@@ -1104,9 +1101,9 @@ void write_dummy(short curr, int use_tm_start, int use_tm_end)
 void copy_structures(int dest, int src, int stype)
 {
    memcpy(&file_stats[dest], &file_stats[src], FILE_STATS_SIZE);
-   if (file_hdr.sa_proc > 0)
+   if (file_hdr.sa_proc)
       memcpy(st_cpu[dest], st_cpu[src],
-	     STATS_ONE_CPU_SIZE * (file_hdr.sa_proc + 1));
+	     STATS_ONE_CPU_SIZE * file_hdr.sa_proc);
    if (GET_ONE_IRQ(file_hdr.sa_actflag))
       memcpy(interrupts[dest], interrupts[src],
 	     STATS_ONE_IRQ_SIZE);
@@ -1118,7 +1115,7 @@ void copy_structures(int dest, int src, int stype)
 	     STATS_SERIAL_SIZE * file_hdr.sa_serial);
    if (file_hdr.sa_irqcpu)
       memcpy(st_irq_cpu[dest], st_irq_cpu[src],
-	     STATS_IRQ_CPU_SIZE * (file_hdr.sa_proc + 1) * file_hdr.sa_irqcpu);
+	     STATS_IRQ_CPU_SIZE * file_hdr.sa_proc * file_hdr.sa_irqcpu);
    if (file_hdr.sa_iface)
       memcpy(st_net_dev[dest], st_net_dev[src],
 	     STATS_NET_DEV_SIZE * file_hdr.sa_iface);
@@ -1135,9 +1132,9 @@ void copy_structures(int dest, int src, int stype)
  */
 void read_extra_stats(short curr, int ifd)
 {
-   if (file_hdr.sa_proc > 0)
+   if (file_hdr.sa_proc)
       sa_fread(ifd, st_cpu[curr],
-	       STATS_ONE_CPU_SIZE * (file_hdr.sa_proc + 1), HARD_SIZE);
+	       STATS_ONE_CPU_SIZE * file_hdr.sa_proc, HARD_SIZE);
    if (GET_ONE_IRQ(file_hdr.sa_actflag))
       sa_fread(ifd, interrupts[curr],
 	       STATS_ONE_IRQ_SIZE, HARD_SIZE);
@@ -1146,7 +1143,7 @@ void read_extra_stats(short curr, int ifd)
 	       STATS_SERIAL_SIZE * file_hdr.sa_serial, HARD_SIZE);
    if (file_hdr.sa_irqcpu)
       sa_fread(ifd, st_irq_cpu[curr],
-	       STATS_IRQ_CPU_SIZE * (file_hdr.sa_proc + 1) * file_hdr.sa_irqcpu, HARD_SIZE);
+	       STATS_IRQ_CPU_SIZE * file_hdr.sa_proc * file_hdr.sa_irqcpu, HARD_SIZE);
    if (file_hdr.sa_iface)
       sa_fread(ifd, st_net_dev[curr],
 	       STATS_NET_DEV_SIZE * file_hdr.sa_iface, HARD_SIZE);
@@ -1159,15 +1156,15 @@ void read_extra_stats(short curr, int ifd)
 
 /*
  ***************************************************************************
- * Read a bunch of statistics sent by the data collector
+ * Read a bunch of statistics sent by the data collector (sadc)
  ***************************************************************************
  */
 void read_stat_bunch(short curr)
 {
    if (sa_read(&file_stats[curr], file_hdr.sa_st_size))
       exit(0);
-   if ((file_hdr.sa_proc > 0) &&
-       sa_read(st_cpu[curr], STATS_ONE_CPU_SIZE * (file_hdr.sa_proc + 1)))
+   if ((file_hdr.sa_proc) &&
+       sa_read(st_cpu[curr], STATS_ONE_CPU_SIZE * file_hdr.sa_proc))
       exit(0);
    if (GET_ONE_IRQ(file_hdr.sa_actflag) &&
        sa_read(interrupts[curr], STATS_ONE_IRQ_SIZE))
@@ -1179,7 +1176,7 @@ void read_stat_bunch(short curr)
        sa_read(st_serial[curr], STATS_SERIAL_SIZE * file_hdr.sa_serial))
       exit(0);
    if (file_hdr.sa_irqcpu &&
-       sa_read(st_irq_cpu[curr], STATS_IRQ_CPU_SIZE * (file_hdr.sa_proc + 1) * file_hdr.sa_irqcpu))
+       sa_read(st_irq_cpu[curr], STATS_IRQ_CPU_SIZE * file_hdr.sa_proc * file_hdr.sa_irqcpu))
       exit(0);
    if (file_hdr.sa_iface &&
        sa_read(st_net_dev[curr], STATS_NET_DEV_SIZE * file_hdr.sa_iface))
@@ -1196,7 +1193,7 @@ void read_stat_bunch(short curr)
  ***************************************************************************
  */
 void handle_curr_act_stats(int ifd, off_t fpos, short *curr, long *cnt, int *eosaf,
-			 int rows, unsigned int act, int *reset)
+			   int rows, unsigned int act, int *reset)
 {
    short dis = 1;
    unsigned long lines;
@@ -1304,7 +1301,7 @@ void read_stats_from_file(char from_file[])
    allocate_structures(USE_SA_FILE);
 
    /* Print report header */
-   print_report_hdr(S_O_NONE, flags, &loc_time, &file_hdr);
+   print_report_hdr(flags, &loc_time, &file_hdr);
 
    /* Read system statistics from file */
    do {
@@ -1450,13 +1447,13 @@ void read_stats(void)
    allocate_structures(USE_SADC);
 
    /* Print report header */
-   print_report_hdr(S_O_NONE, flags, &loc_time, &file_hdr);
+   print_report_hdr(flags, &loc_time, &file_hdr);
 
    /* Read system statistics sent by the data collector */
    read_stat_bunch(0);
 
    if (!dis_hdr) {
-      if (file_hdr.sa_proc > 0)
+      if (file_hdr.sa_proc)
 	 more = 2 + file_hdr.sa_proc;
       if (pid_nr)
 	 more = pid_nr;
@@ -1517,6 +1514,9 @@ int main(int argc, char **argv)
    char from_file[MAX_FILE_LEN], to_file[MAX_FILE_LEN];
    char ltemp[20];
 
+   /* Get HZ */
+   get_HZ();
+
    /* Compute page shift in kB */
    kb_shift = get_kb_shift();
 
@@ -1571,12 +1571,8 @@ int main(int argc, char **argv)
 	    strncpy(from_file, argv[opt++], MAX_FILE_LEN);
 	    from_file[MAX_FILE_LEN - 1] = '\0';
 	 }
-	 else {
-	    get_localtime(&loc_time);
-	    snprintf(from_file, MAX_FILE_LEN,
-		     "%s/sa%02d", SA_DIR, loc_time.tm_mday);
-	    from_file[MAX_FILE_LEN - 1] = '\0';
-	 }
+	 else
+	    set_default_file(&loc_time, from_file);
       }
 
       else if (!strcmp(argv[opt], "-s")) {
@@ -1692,12 +1688,8 @@ int main(int argc, char **argv)
 
    /* 'sar' is equivalent to 'sar -f' */
    if ((argc == 1) ||
-       ((interval < 0) && !from_file[0] && !to_file[0])) {
-      get_localtime(&loc_time);
-      snprintf(from_file, MAX_FILE_LEN,
-	       "%s/sa%02d", SA_DIR, loc_time.tm_mday);
-      from_file[MAX_FILE_LEN - 1] = '\0';
-   }
+       ((interval < 0) && !from_file[0] && !to_file[0]))
+      set_default_file(&loc_time, from_file);
 
    /*
     * Check option dependencies
